// Kingdom of Kod
// ==============
// -Adventures of the heroes in the 'Manga Guide to Databases'

// -Filename: test_basic_sanity.cpp
// -The definitions of a few basic type member tests for the system

// doctest tutorial
// https://github.com/onqtam/doctest/blob/master/doc/markdown/tutorial.md

#include <vector>

#include "testing/doctest.h"

#include "Action.hpp"
#include "Camera.hpp"
#include "Character.hpp"
#include "Dialog.hpp"
#include "Effect.hpp"
#include "Environment.hpp"
#include "Human.hpp"
#include "Light.hpp"
#include "Named_3D.hpp"
#include "Prod_log.hpp"
#include "Production.hpp"
#include "Prop.hpp"
#include "Robowaifu.hpp"
#include "Scene.hpp"
#include "Script.hpp"
#include "Sequence.hpp"
#include "Shot.hpp"
#include "Timeline.hpp"

//------------------------------------------------------------------------------
// Perform some simple type member tests to confirm the architecture for derived
// types & base class members is working OK; just do a few basic sanity checks.
//
TEST_CASE(
    "A vector of Named_3D objects can be created via the derived type's "
    "default ctors") {
  //
  std::vector<Named_3D> all_objs{
      Action{}, Camera{}, Character{}, Dialog{},     Effect{},  Environment{},
      Human{},  Light{},  Prod_log{},  Production{}, Prop{},    Robowaifu{},
      Scene{},  Script{}, Sequence{},  Shot{},       Timeline{}};
  REQUIRE(all_objs.size() == 17);

  // confirm each object's fields are both initialized & blank
  const Posn foo_posn{};  // a fully instantiated, and known-blank, Posn
  //
  for (const auto& e : all_objs) {
    REQUIRE(e.name() == "");
    REQUIRE(e.type_tag() == "");
    REQUIRE(e.posn() == foo_posn);
  }

  auto& action      = all_objs[0];
  auto& camera      = all_objs[1];
  auto& character   = all_objs[2];
  auto& dialog      = all_objs[3];
  auto& effect      = all_objs[4];
  auto& environment = all_objs[5];
  auto& human       = all_objs[6];
  auto& light       = all_objs[7];
  auto& prod_log    = all_objs[8];
  auto& production  = all_objs[9];
  auto& prop        = all_objs[10];
  auto& robowaifu   = all_objs[11];
  auto& scene       = all_objs[12];
  auto& script      = all_objs[13];
  auto& sequence    = all_objs[14];
  auto& shot        = all_objs[15];
  auto& timeline    = all_objs[16];

  ////////  NAME
  SUBCASE(
      "A contained object's name can be assigned to and read from, following "
      "construction") {
    //
    action.name("action");
    camera.name("camera");
    character.name("character");
    dialog.name("dialog");
    effect.name("effect");
    environment.name("environment");
    human.name("human");
    light.name("light");
    prod_log.name("prod_log");
    production.name("production");
    prop.name("prop");
    robowaifu.name("robowaifu");
    scene.name("scene");
    script.name("script");
    sequence.name("sequence");
    shot.name("shot");
    timeline.name("timeline");

    REQUIRE(action.name() == "action");
    REQUIRE(camera.name() == "camera");
    REQUIRE(character.name() == "character");
    REQUIRE(dialog.name() == "dialog");
    REQUIRE(effect.name() == "effect");
    REQUIRE(environment.name() == "environment");
    REQUIRE(human.name() == "human");
    REQUIRE(light.name() == "light");
    REQUIRE(prod_log.name() == "prod_log");
    REQUIRE(production.name() == "production");
    REQUIRE(prop.name() == "prop");
    REQUIRE(robowaifu.name() == "robowaifu");
    REQUIRE(scene.name() == "scene");
    REQUIRE(script.name() == "script");
    REQUIRE(sequence.name() == "sequence");
    REQUIRE(shot.name() == "shot");
    REQUIRE(timeline.name() == "timeline");
  }

  ////////  TYPE
  SUBCASE(
      "A contained object's type tag can be assigned to and read from, "
      "following construction") {
    //
    action.type_tag("Action");
    camera.type_tag("Camera");
    character.type_tag("Character");
    dialog.type_tag("Dialog");
    effect.type_tag("Effect");
    environment.type_tag("Environment");
    human.type_tag("Human");
    light.type_tag("Light");
    prod_log.type_tag("Prod_log");
    production.type_tag("Production");
    prop.type_tag("Prop");
    robowaifu.type_tag("Robowaifu");
    scene.type_tag("Scene");
    script.type_tag("Script");
    sequence.type_tag("Sequence");
    shot.type_tag("Shot");
    timeline.type_tag("Timeline");

    REQUIRE(action.type_tag() == "Action");
    REQUIRE(camera.type_tag() == "Camera");
    REQUIRE(character.type_tag() == "Character");
    REQUIRE(dialog.type_tag() == "Dialog");
    REQUIRE(effect.type_tag() == "Effect");
    REQUIRE(environment.type_tag() == "Environment");
    REQUIRE(human.type_tag() == "Human");
    REQUIRE(light.type_tag() == "Light");
    REQUIRE(prod_log.type_tag() == "Prod_log");
    REQUIRE(production.type_tag() == "Production");
    REQUIRE(prop.type_tag() == "Prop");
    REQUIRE(robowaifu.type_tag() == "Robowaifu");
    REQUIRE(scene.type_tag() == "Scene");
    REQUIRE(script.type_tag() == "Script");
    REQUIRE(sequence.type_tag() == "Sequence");
    REQUIRE(shot.type_tag() == "Shot");
    REQUIRE(timeline.type_tag() == "Timeline");
  }

  ////////  POSN
  SUBCASE(
      "A contained object's posn can be assigned to and read from, following "
      "construction") {
    //
    action.posn(1, 2, 3);
    camera.posn(4, 5, 6);
    character.posn(7, 8, 9);
    dialog.posn(10, 11, 12);
    effect.posn(13, 14, 15);
    environment.posn(16, 17, 18);
    human.posn(19, 20, 21);
    light.posn(22, 23, 24);
    prod_log.posn(25, 26, 27);
    production.posn(28, 29, 30);
    prop.posn(31, 32, 33);
    robowaifu.posn(34, 35, 36);
    scene.posn(37, 38, 39);
    script.posn(40, 41, 42);
    sequence.posn(43, 44, 45);
    shot.posn(46, 47, 48);
    timeline.posn(49, 50, 51);

    REQUIRE(action.posn() == Posn{1, 2, 3});
    REQUIRE(camera.posn() == Posn{4, 5, 6});
    REQUIRE(character.posn() == Posn{7, 8, 9});
    REQUIRE(dialog.posn() == Posn{10, 11, 12});
    REQUIRE(effect.posn() == Posn{13, 14, 15});
    REQUIRE(environment.posn() == Posn{16, 17, 18});
    REQUIRE(human.posn() == Posn{19, 20, 21});
    REQUIRE(light.posn() == Posn{22, 23, 24});
    REQUIRE(prod_log.posn() == Posn{25, 26, 27});
    REQUIRE(production.posn() == Posn{28, 29, 30});
    REQUIRE(prop.posn() == Posn{31, 32, 33});
    REQUIRE(robowaifu.posn() == Posn{34, 35, 36});
    REQUIRE(scene.posn() == Posn{37, 38, 39});
    REQUIRE(script.posn() == Posn{40, 41, 42});
    REQUIRE(sequence.posn() == Posn{43, 44, 45});
    REQUIRE(shot.posn() == Posn{46, 47, 48});
    REQUIRE(timeline.posn() == Posn{49, 50, 51});
  }
}

// Copyright (2021)
// License MIT (Expat) https://opensource.org/licenses/MIT
